/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 */
 
package org.exolab.tools.ant;


import java.io.File;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.Path;


/**
 * Task to generate Java sources from an ANTLR grammar. 
 * <br>
 * This task accepts the following arguments:
 * <p>
 * <table border="0" cellpadding="1" cellspacing="3" >
 * <tr valign=top><td>grammar</td><td>specifies the path to the ANTLR grammar
 *     </td></tr>
 * <tr valign=top><td>dependencies</td><td>a list of files that the grammar
 *     is dependent on</td></tr>
 * <tr valign=top><td>targets</td><td>a list of class names produced from
 *     <b>grammar</b></td></tr>
 * <tr valign=top><td>optDir</td><td>the directory to write the build options 
 *     (<i>&lt;grammar&gt;.opt</i>) file to. This is used:
 *     <ul> 
 *     <li> to determine if any of the targets need to be rebuilt. 
 *     <li> by <b>clean-targets</b> to determine what files to remove.
 *     </ul>
 *     If not set, the options file will be written to the same directory as 
 *     <b>grammar</b>.<br>
 *     In the latter case, the <b>javac</b> task will copy these to its 
 *     <i>destdir</i> property; to avoid this the <b>javac</b> <i>excludes</i> 
 *     property should be set filter out <i>.opt</i> files.
 * <tr valign=top><td>exportVocab</td><td>specifies the name of the exported 
 *     vocabulary - this adds the related token types files as targets
 *     </td></tr>
 * <tr valign=top><td>debug</td><td>when true, launch the ParseView debugger 
 *     upon parser invocation </td></tr>
 * <tr valign=top><td>trace</td><td>when true, have all rules call 
 *     traceIn/traceOut</td></tr>
 * <tr valign=top><td>traceLexer</td><td>when true, have lexer rules call 
 *     traceIn/traceOut</td></tr>
 * <tr valign=top><td>traceParser</td><td>when true, have parser rules call 
 *     traceIn/traceOut</td></tr>
 * <tr valign=top><td>traceTreeParser</td><td>when true, have tree parser 
 *     rules call traceIn/traceOut</td></tr>
 * </table>
 * <p>
 * Of these arguments, <b>grammar</b> and <b>targets</b> are required.
 * <p>
 *
 * @version     $Revision: 1.1.1.1 $ $Date: 2002/01/22 00:21:36 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         org.exolab.tools.ant.CleanTargets
 */
public class Antlr extends Java {

    /**
     * The ANTLR build rule
     */
    private AntlrBuildRule _antlrRule = new AntlrBuildRule();

    /**
     * The directory to write the <grammar>.opt build rule to
     */
    private String _optDir = null;

    /**
     * Set the path to the ANTLR grammar file
     *
     * @param path the path to the grammar file
     */
    public void setGrammar(String path) {
        _antlrRule.setGrammar(path);
    }

    /**
     * Set the list of files that the grammar is dependent on
     *
     * @param dependencies a comma or space separated list of paths
     */
    public void setDependencies(String dependencies) {
        _antlrRule.setDependencies(dependencies);
    }

    /**
     * Set the list of classes produced by the grammar
     *
     * @param targets a comma or space separated list of class names
     */
    public void setTargets(String targets) {
        _antlrRule.setTargets(targets);
    }

    /**
     * Set the directory to write the <grammar>.opt build rule to.
     *
     * @param path the directory to write the <grammar>.opt file to.
     */
    public void setOptDir(String path) {
        _optDir = path;
    }

    /**
     * Set the name of vocabulary that the grammar exports
     *
     * @param vocab the name that the grammar exports
     */
    public void setExportVocab(String vocab) {
        _antlrRule.setExportVocab(vocab);
    }

    /**
     * If true, launch the ParseView debugger upon parser invocation
     *
     * @param debug if "true" or "yes", launch the ParseView debugger 
     * when the parser is invoked.
     */
    public void setDebug(String debug) {
	_antlrRule.setDebug(Project.toBoolean(debug));
    }

    /**
     * If true, have all rules call traceIn/traceOut
     *
     * @param trace if "true" or "yes" turn on debug tracing
     */
    public void setTrace(String trace) {
        _antlrRule.setTrace(Project.toBoolean(trace));
    }

    /**
     * If true, have lexer rules call traceIn/traceOut
     *
     * @param trace if "true" or "yes" turn on lexer debug tracing
     */
    public void setTraceLexer(String trace) {
        _antlrRule.setTraceLexer(Project.toBoolean(trace));
    }

    /**
     * If true, have parser rules call traceIn/traceOut
     *
     * @param trace if "true" or "yes" turn on parser debug tracing
     */
    public void setTraceParser(String trace) {
        _antlrRule.setTraceParser(Project.toBoolean(trace));
    }

    /**
     * If true, have tree parser rules call traceIn/traceOut
     *
     * @param trace if "true" or "yes" turn on tree parser debug tracing
     */
    public void setTraceTreeParser(String trace) {
        _antlrRule.setTraceTreeParser(Project.toBoolean(trace));
    }

    /**
     * Execute this task
     *
     * @throws BuildException if the parser generation fails
     */
    public void execute() 
	throws BuildException {

        BuildRule bld = _antlrRule.create();
	File source = project.resolveFile(_antlrRule.getGrammar());
        
        String dir = source.getParent();
        setDir(new File(dir));      // need to set the grammar directory 
                                    // so ANTLR can find importVocabs
	setFork(true);              // ANTLR calls System.exit()

	setClasspath(Path.systemClasspath);
	if(File.separator.equals("\\")) {
            createJvmarg().setValue("\"-Duser.dir=" + 
                                    System.getProperty("user.dir") + "\"");
        } else {
            createJvmarg().setValue("-Duser.dir=" + 
                                    System.getProperty("user.dir"));
        }
        // required due to setDir() above - this enables relative class
        // paths to work

	setClassname("antlr.Tool");

        String path = getRulePath(bld);
	if (BuildRuleHelper.rebuild(bld, path, project)) {
	    createArg().setLine(_antlrRule.getArgs());
            super.execute();

            BuildRuleHelper.log(bld, path);
            BuildRuleHelper.checkTargets(bld);
        }
    }

    /**
     * Get the path to write build rule file to
     *
     * @return the path to write build rule file to
     */
    private String getRulePath(BuildRule rule) {

        String path = null;
        File source = new File(_antlrRule.getGrammar());
        if (_optDir != null) {
            path = rule.path(_optDir, source.getName());
        } else {
            path = rule.path(source.getParent(), source.getName());
        }

        return path;
    }

} // End Antlr


/**
 * Helper class to instantiate a {@link BuildRule} instance for an ANTLR
 * target
 */
class AntlrBuildRule {

    /**
     * The path of the ANTLR grammar file
     */
    private String _grammar = null;

    /**
     * A comma or space separated list of files that the grammar is dependent
     * on
     */
    private String _dependencies = null;

    /**
     * A comma or space separated list of class names that the grammar produces
     */
    private String _targets = null;

    /**
     * The vocabulary that the grammar exports (if any)
     */
    private String _exportVocab = null;

    /**
     * If true, launch the ParseView debugger upon parser invocation
     */
    private boolean _debug = false; 

    /**
     * If true, have all rules call traceIn/traceOut
     */
    private boolean _trace = false;

    /**
     * If true, have lexer rules call traceIn/traceOut
     */
    private boolean _traceLexer = false;

    /**
     * If true, have parser rules call traceIn/traceOut
     */
    private boolean _traceParser = false;

    /**
     * If true, have tree parser rules call traceIn/traceOut
     */
    private boolean _traceTreeParser = false;

    /**
     * Option names used when invoking {@link BuildRule#addOption}
     */
    private static final String DEBUG = "debug";
    private static final String TRACE = "trace";
    private static final String TRACE_LEXER = "traceLexer";
    private static final String TRACE_PARSER = "traceParser";
    private static final String TRACE_TREE_PARSER = "traceTreeParser";

    /**
     * File name extensions for files generated when a vocabulary is    
     * exported
     */
    private static final String TOKEN_JAVA_EXT = "TokenTypes.java";
    private static final String TOKEN_TEXT_EXT = "TokenTypes.txt";

    /**
     * Set the path to the ANTLR grammar file
     *
     * @param path the path to the grammar file
     */
    public void setGrammar(String path) {
        _grammar = path;
    }

    /**
     * Return the path to the ANTLR grammar file
     *
     * @param the path to the grammar file
     */
    public String getGrammar() {
        return _grammar;
    }

    /**
     * Set the list of files that the grammar is dependent on
     *
     * @param dependencies a comma or space separated list of paths
     */
    public void setDependencies(String dependencies) {
        _dependencies = dependencies;
    }

    /**
     * Set the list of classes produced by the grammar
     *
     * @param targets a comma or space separated list of class names
     */
    public void setTargets(String targets) {
        _targets = targets;
    }

    /**
     * Set the name of vocabulary that the grammar exports
     *
     * @param vocab the name that the grammar exports
     */
    public void setExportVocab(String value) {
        _exportVocab = value;
    }

    /**
     * If true, launch the ParseView debugger upon parser invocation
     *
     * @param debug if <code>true</code> launch the ParseView debugger 
     * when the parser is invoked.
     */
    public void setDebug(boolean debug) {
        _debug = debug;
    }

    /**
     * If true, have all rules call traceIn/traceOut
     *
     * @param trace if <code>true</code> turn on debug tracing
     */
    public void setTrace(boolean trace) {
        _trace = trace;
    }

    /**
     * If true, have lexer rules call traceIn/traceOut
     *
     * @param trace if <code>true</code> turn on lexer debug tracing
     */
    public void setTraceLexer(boolean trace) {
        _traceLexer = trace;
    }

    /**
     * If true, have parser rules call traceIn/traceOut
     *
     * @param trace if <code>true</code> turn on parser debug tracing
     */
    public void setTraceParser(boolean trace) {
        _traceParser = trace;
    }

    /**
     * If true, have tree parser rules call traceIn/traceOut
     *
     * @param trace if <code>true</code> turn on parser debug tracing
     */
    public void setTraceTreeParser(boolean trace) {
        _traceTreeParser = trace;
    }

    /**
     * Create a {@link BuildRule} instance for an ANTLR target
     *
     * @return the <code>BuildRule<code> for the ANTLR target
     * @throws BuildException if the grammar of targets have not been set
     */
    public BuildRule create()
        throws BuildException {

        if (_grammar == null) {
            throw new BuildException("grammar not specified");
        }

        if (_targets == null) {
            throw new BuildException("targets not specified");
        }

        BuildRule rule = new BuildRule();

        String dir = new File(_grammar).getParent();
        
        // add dependencies
        if (_dependencies != null) {
            StringTokenizer paths = new StringTokenizer(_dependencies, ", ");
            while (paths.hasMoreTokens()) {
                String path = paths.nextToken().trim();
                rule.addDependency(path);
            }
        }
 
        // add sources
        rule.addSource(_grammar);
        
        // add targets
        StringTokenizer targets = new StringTokenizer(_targets, ", ");
        while (targets.hasMoreTokens()) {
            String name = targets.nextToken().trim();
            rule.addTarget(new File(dir, name + ".java").getPath());
        }

        // if a vocab is exported, add the generated targets
        if (_exportVocab != null) {
            String path = null;
            path = new File(dir, _exportVocab + TOKEN_JAVA_EXT).getPath();
            rule.addTarget(path);
            path = new File(dir, _exportVocab + TOKEN_TEXT_EXT).getPath();
            rule.addTarget(path);
        }

        // add options
        rule.addOption(DEBUG, new Boolean(_debug));
        rule.addOption(TRACE, new Boolean(_trace));
        rule.addOption(TRACE_LEXER, new Boolean(_traceLexer));
        rule.addOption(TRACE_PARSER, new Boolean(_traceParser));
        rule.addOption(TRACE_TREE_PARSER, new Boolean(_traceTreeParser));
        
        return rule;
    }

    /**
     * Returns the command line arguments to be passed to antlr.Tool
     * to build the targets.
     *
     * @return the antlr.Tool command line arguments
     */
    public String getArgs() {

	StringBuffer args = new StringBuffer();
        if (_debug) {
	    args.append("-debug ");
	}
        if (_trace) {
	    args.append("-trace ");
        }
        if (_traceLexer) {
	    args.append("-traceLexer ");
        }
        if (_traceParser)
        {
	    args.append("-traceParser ");
        }
        if (_traceTreeParser)
        {
	    args.append("-traceTreeParser ");
        }

        args.append(new File(_grammar).getName());
	return args.toString();
    }

} // End AntlrBuildRule
