/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 */

package org.exolab.tools.ant;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.HashMap;

import org.apache.tools.ant.BuildException;


/**
 * This class enables tasks to perform make-like checks to determine
 * if one or more targets need to be rebuilt. 
 * <p>
 * It does this by providing methods to track dependencies, source, target 
 * and build options required by the task.
 * <p>
 * Instances may be made persistent so that they can be compared with
 * subsequent builds to force a rebuild if any of the details have changed
 * (eg, an optional property used by the task)
 *
 * @version     $Revision: 1.1.1.1 $ $Date: 2002/01/22 00:21:37 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         org.exolab.tools.ant.BuildRuleHelper
 * @see         org.exolab.tools.ant.CleanTargets
 */
class BuildRule 
    implements Serializable {

    /**
     * The list of source files
     */
    private List _sources = new ArrayList();
    
    /**
     * The list of dependencies of the source files
     */
    private List _dependencies = new ArrayList();

    /**
     * The list of targets produced from the source files
     */
    private List _targets = new ArrayList();

    /**
     * The build options
     */
    private HashMap _options = new HashMap();

    /**
     * Add a dependency path
     *
     * @param path the dependency path
     */
    public void addDependency(String path) {
        _dependencies.add(path);
    }

    /**
     * Return the list of dependencies
     *
     * @return the list of dependencies
     */
    public List getDependencies() {
        return _dependencies;
    }

    /**
     * Add a source path
     *
     * @param path the source path
     */
    public void addSource(String path) {
        _sources.add(path);
    }
       
    /**
     * Return the list of sources
     *
     * @return the list of sources
     */
    public List getSources() {
        return _sources;
    }

    /**
     * Add a target path
     *
     * @param path the target path
     */
    public void addTarget(String path) {
        _targets.add(path);
    }

    /**
     * Return the list of targets
     *
     * @return the list of targets
     */
    public List getTargets() {
        return _targets;
    }

    /**
     * Add a build option
     *
     * @param name the name of the option
     * @param value the value of the option
     */
    public void addOption(String name, Object value) {
        _options.put(name, value);
    }

    /**
     * Return the build options
     *
     * @return the build options
     */
    public HashMap getOptions() {
        return _options;
    }
    
    /**
     * Returns true if the dependencies are newer than the sources,
     * or the sources are newer than the targets
     *
     * @return <code>true</code> if this rule needs a rebuild
     * @throws BuildException if a source of dependency file cannot be found
     */
    public boolean rebuild()
        throws BuildException {

        boolean result = false;
        if (modified(_dependencies, _sources) || 
            modified(_sources, _targets)) {
            result = true;
        }
        
        return result;
    }

    /**
     * Read a build rule from file
     *
     * @param path the build rule path
     * @return the build rule, or <code>null</code>, if it can't be found
     */
    public static BuildRule read(String path) {
        BuildRule result = null;
	ObjectInputStream stream = null;
	try {
	    stream = new ObjectInputStream(new FileInputStream(path));
            result = (BuildRule) stream.readObject();
	} catch (Exception ignore) {
	} finally {
            try {
                if (stream != null) {
                    stream.close();
                }
            } catch (IOException ignore) {
            }
        }
        return result;
    }

    /**
     * Write a build rule to file
     *
     * @param rule the rule to write
     * @param path the path to write to
     * @return <code>true</code> if the file is successfully written
     */
    public static boolean write(BuildRule rule, String path) {
	ObjectOutputStream stream = null;
        boolean success = false;
	try {
	    stream = new ObjectOutputStream(new FileOutputStream(path));
	    stream.writeObject(rule);
            success = true;
	} catch (IOException ignore) {
        } finally {
	    try {
                if (stream != null) {
                    stream.close();
                }
	    } catch (IOException ignore) {
	    }
	}
        return success;
    }

    /**
     * Compare this with another object for equality
     *
     * @return <code>true</code> if this is equivalent to the other object
     */
    public boolean equals(Object object) {
	boolean equal = false;
	if (object instanceof BuildRule) {
            BuildRule rhs = (BuildRule) object;
            if (_dependencies.equals(rhs._dependencies) &&
                _sources.equals(rhs._sources) &&
                _targets.equals(rhs._targets) &&
                _options.equals(rhs._options)) {
                equal = true;
            }
	}
	return equal;
    }

    /**
     * Return the default name for this build rule.     
     * <p>
     * This is derived from the first source file path. If none is set,
     * <code>null</code> is returned.
     * 
     * @return the default name for the build rule, or <code>null</code>, if
     * it cannot be determined
     */
    public String defaultName() {
        String name = null;
        if (!_sources.isEmpty()) {
            name = defaultName(new File((String) _sources.get(0)).getName());
        }
        return name;
    }

    /**
     * Return the default name for a build rule given a source path
     *
     * @param source the source file name
     * @return the build rule name
     */
    public static String defaultName(String source) {
        return source + ".opt";
    }

    /**
     * Return the path of a build rule file
     *
     * @param dir the build rule directory path
     * @param source the source file name
     * @return the build rule path
     */
    public static String path(String dir, String source) {
        return new File(dir, defaultName(source)).getPath();
    }

    /**
     * Returns <code>true</code> if any of the sources have been modified
     * since the targets were produced.
     *
     * @param sources the list of sources
     * @param targets the list of targets
     * @return <code>true</code> if any of the sources have been modified 
     * since the targets were produced
     * @throws BuildException if a source file cannot be located
     */
    protected boolean modified(List sources, List targets)
        throws BuildException {

        boolean result = false;
        Iterator srcIter = sources.iterator();
        while (!result && srcIter.hasNext()) {
            File source = new File((String) srcIter.next());
            Iterator tgtIter = targets.iterator();
            while (tgtIter.hasNext()) {
                File target = new File((String) tgtIter.next());
                if (modified(source, target)) {
                    result = true;
                    break;
                }
            }
        }
        return result;
    }

    /** 
     * Returns <code>true</code> if a source file has been modified since
     * its target was produced, or the target doesn't exist
     *
     * @param source the source file
     * @param target the target file
     * @throws BuildException if the source file doesn't exist
     */
    protected boolean modified(File source, File target)
        throws BuildException {

        boolean result = false;
        if (!source.exists()) {
            throw new BuildException("Source does not exist: " + 
                                     source.getPath());
        }
	if (!target.exists() || source.lastModified() > target.lastModified())
	{
	    result = true;
	}
        return result;
    }

}  // End BuildRule
