/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 */

package org.exolab.tools.ant;


import java.io.File;
import java.util.Iterator;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;

/**
 * This class provides helper methods for operating on instances of 
 * BuildRule.
 *
 * @version     $Revision: 1.1.1.1 $ $Date: 2002/01/22 00:21:37 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         org.exolab.tools.ant.BuildRule
 */
class BuildRuleHelper
{
    /**
     * Returns true if the targets generated from the source(s) need to be 
     * rebuilt, the rule has changed since the targets were generated,
     * or no prior build rule exists.
     *
     * @param rule the build rule
     * @param path the build rule dependency file path
     * @param project the ANT project, for logging purposes
     * @return <code>true</code> if the target needs to be rebuilt
     * @throws BuildException if the source cannot be located
     */
    public static boolean rebuild(
        BuildRule rule, String path, Project project) 
        throws BuildException {

        boolean result = false;
        BuildRule previous = BuildRule.read(path);

        if (!rule.equals(previous)) {
            // rebuild if a prior rule does not exist, or the rule has changed
            String source = (!rule.getSources().isEmpty()) 
                ? (String) rule.getSources().get(0) : "<unknown>";
            if (previous == null) {
                project.log("Targets of " + source + 
                            " need rebuilding: " + path + 
                            " does not exist", project.MSG_VERBOSE);
            }
            else {
                project.log("Targets of " + source + 
                            " need rebuilding: the build rule has changed",
                            project.MSG_VERBOSE);
            }
            result = true;
        }
        else {
            result = rebuild(rule, project);
        }
        return result;
    }

    /**
     * Returns true if the target specified by the rule needs to be rebuilt
     *
     * @param rule the build rule
     * @param project the ANT project, for logging purposes
     * @return <code>true</code> if the target needs to be rebuilt
     * @throws BuildException if the source cannot be located
     */
    public static boolean rebuild(BuildRule rule, Project project)
        throws BuildException {

	boolean result = rule.rebuild();
        if (result) {
            String source = (!rule.getSources().isEmpty())
                ? (String) rule.getSources().get(0) : "<unknown>";
            project.log("Targets of " + source + 
                        " need rebuilding: targets are out of date, or " +
                        "don't exist", project.MSG_VERBOSE);
        }
	return result;
    }

    /**
     * Verify that all required targets have been generated
     *
     * @param rule the build rule
     * @throws BuildException if a specified target was not generated
     */
    public static void checkTargets(BuildRule rule)
        throws BuildException {

        Iterator iter = rule.getTargets().iterator();
        while (iter.hasNext())
        {
            File target = new File((String) iter.next());
            if (!target.exists()) {
                throw new BuildException("Target was not created: " + 
                                         target.getPath());
            }
        }
    }

    /**
     * Log the build rule, so it can be compared against that of subsequent
     * builds.
     *
     * @param rule the build rule
     * @param path the path to write the build rule to
     * @throws BuildException if the build rule cannot be written
     */
    public static void log(BuildRule rule, String path)
        throws BuildException
    {
        if (!BuildRule.write(rule, path)) {
            throw new BuildException("Failed to write build rule to " + path);
        }
    }

    /**
     * Cleans all targets
     *
     * @param rule the rule to clean all targets for
     * @param project the ANT project, for logging purposes
     */
    public static void clean(BuildRule rule, Project project) {

        Iterator iter = rule.getTargets().iterator();
        while (iter.hasNext()) {
            File target = new File((String) iter.next());
            if (target.exists()) {
                project.log("Deleting: " + target.getPath(), 
                            project.MSG_VERBOSE);
                target.delete();
            } else {
                project.log("Cannot delete: " + target.getPath() +
                            ", target does not exist", project.MSG_VERBOSE);
            }
        }
    }

} // End BuildRuleHelper
