/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 */
 
package org.exolab.tools.ant;


import java.io.File;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Java;


/**
 * Task to generate Java sources from an XML schema using Castor. 
 * <br>
 * This task accepts the following arguments:
 * <p>
 * <table border="0" cellpadding="1" cellspacing="3">
 * <tr><td>schema</td><td>specifies the path to the XML schema</td></tr>
 * <tr><td>targets</td><td>a list of class names produced from <b>schema</b>
 *     </td></tr>
 * <tr><td>package</td><td>specifies the package of the generated classes
 *     </td></tr>
 * <tr><td>destDir</td><td>the destination directory for targets. This must
 *     be used in conjunction with <b>package</b>
 *     </td></tr>
 * <tr valign=top><td>types</td><td>specifies the collection types used by 
 *     Castor. 
 *     These are:
 *     <ul>
 *     <li><b>j1</b> - Java 1.1: java.util.Vector. This is the default
 *     <li><b>j2</b> - Java 1.2: java.util.Collection
 *     <li><b>odmg</b> - ODMG 3.0: odmg.DArray
 *     </ul>
 *     </td></tr>
 * <tr valign=top><td>optDir</td><td>the directory to write the build options 
 *     (<i>&lt;schema&gt;.opt</i>) file to. This is used:
 *     <ul> 
 *     <li> to determine if any of the targets need to be rebuilt. 
 *     <li> by <b>clean-targets</b> to determine what files to remove.
 *     </ul>
 *     If not set, the options file will be written to the same directory as 
 *     <b>schema</b>.<br>
 *     In the latter case, the <b>javac</b> task will copy these to its 
 *     <i>destdir</i> property; to avoid this the <b>javac</b> <i>excludes</i> 
 *     property should be set filter out <i>.opt</i> files.
 * </table>
 * <p>
 * Of these arguments, <b>schema</b> and <b>targets</b> are required.
 * <p>
 *
 * @version     $Revision: 1.2 $ $Date: 2002/01/25 00:04:31 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         org.exolab.tools.ant.CleanTargets
 */
public class CastorSrcGenerator 
    extends Java {

    /**
     * The Castor build rule
     */
    private CastorBuildRule _castorRule = new CastorBuildRule();

    /**
     * The directory to write the <schema>.opt build rule to
     */
    private String _optDir = null;

    /**
     * Set the path to the schema
     * <p>
     * This property is mandatory.
     *
     * @param path the path to the schema
     */
    public void setSchema(String path) {
        _castorRule.setSchema(path);
    }

    /**
     * Set the list of targets produced from the schema. This is a comma or 
     * space separated list of class names
     * <p>
     * This property is mandatory.
     *
     * @param targets a comma or space separated list of class names
     */
    public void setTargets(String targets) {
        _castorRule.setTargets(targets);
    }

    /**
     * Set the package of the generated targets. 
     * <p>
     * This must be used in conjunction with {@link #setDestDir}.
     *
     * @param name the name of the package
     */
    public void setPackage(String name) {
        _castorRule.setPackage(name);
    }

    /**
     * Set the types of java collection classes to produce in the targets
     *
     * @param types the types of java collection classes to produce
     */
    public void setTypes(String types) {
        _castorRule.setTypes(types);
    }

    /**
     * Set the destination directory path to output generated targets
     *
     * @param dir the destination directory to output targets
     */
    public void setDestDir(String dir) {
        _castorRule.setDestDir(dir);
    }

    /**
     * Set the directory to write the <schema>.opt build rule to
     *
     * @param dir the directory to write the build rule file to
     */
    public void setOptDir(String dir) {
        _optDir = dir;
    }

    /**
     * Execute this task
     *
     * @throws BuildException if the target generation fails
     */    
    public void execute() 
	throws BuildException {

        BuildRule bld = _castorRule.create();
	File source = project.resolveFile(_castorRule.getSchema());

	setClassname("org.exolab.castor.builder.SourceGenerator");

        String path = getRulePath(bld);
	if (BuildRuleHelper.rebuild(bld, path, project)) {
	    createArg().setLine(_castorRule.getArgs());
            super.execute();

            BuildRuleHelper.log(bld, path);
            BuildRuleHelper.checkTargets(bld);
        }
    }

    /**
     * Get the path to write build rule file to
     *
     * @return the path to write build rule file to
     */
    private String getRulePath(BuildRule rule) {
        File source = new File(_castorRule.getSchema());
        String path = null;
        if (_optDir != null) {
            path = rule.path(_optDir, source.getName());
        } else {
            path = rule.path(source.getParent(), source.getName());
        }

        return path;
    }

} // End CastorSrcGenerator

/**
 * Helper class to instantiate a {@link BuildRule} instance for a Castor
 * target
 */
class CastorBuildRule {

    /**
     * The path of the XML schema
     */
    private String _schema = null;

    /**
     * A comma or space separated list of class names that the schema produces
     */
    private String _targets = null;

    /**
     * The package of the targets
     */
    private String _targetPackage = null;

    /**
     * The types of java collection classes to use
     */
    private String _types = null;

    /**
     * The directory to output the targets to
     */
    private String _destDir = null;

    /**
     * Option names used when invoking {@link BuildRule#addOption}
     */
    private static final String PACKAGE = "package";
    private static final String TYPES = "types";
    private static final String DEST_DIR = "destDir";

    /**
     * File extensions of generated targets
     */
    private static final String SOURCE_EXT = ".java";
    private static final String DESCRIPTOR_EXT = "Descriptor.java";


    /**
     * Set the schema path
     *
     * @param path the schema path
     */
    public void setSchema(String path) {
        _schema = path;
    }

    /**
     * Return the schema path
     *
     * @return the schema path
     */
    public String getSchema() {
        return _schema;
    }

    /**
     * Set the list of classes produced by the schemea
     *
     * @param targets a comma or space separated list of class names
     */
    public void setTargets(String targets) {
        _targets = targets;
    }

    /**
     * Set the package of the targets
     *
     * @param name the package name
     */    
    public void setPackage(String name) {
        _targetPackage = name;
    }

    /**
     * Set the types of java collection classes to produce in the targets
     *
     * @param types the types of java collection classes to produce
     */
    public void setTypes(String types) {
        _types = types;
    }

    /**
     * Set the destination directory path to output generated targets
     *
     * @param dir the destination directory to output targets
     */
    public void setDestDir(String dir) {
        _destDir = dir;
    }

    /**
     * Create a build rule from the current properties
     *
     * @throws BuildException if 'schema' or 'targets' isn't specified, if
     * the value of 'types' is invalid, or 'destDir' is not used in conjunction
     * with 'package'
     */
    public BuildRule create()
        throws BuildException {

        if (_schema == null) {
            throw new BuildException("'schema' not specified");
        }

        if (_targets == null) {
            throw new BuildException("'targets' not specified");
        }

        if (_types != null && !(_types.equals("j1") || _types.equals("j2") ||
                                _types.equals("odmg"))) {
            throw new BuildException("invalid value for 'types': " + _types);
        }

        if (_destDir != null && _targetPackage == null) {
            throw new BuildException("'destDir' must be used in conjunction " +
                                     "with 'package'");
        }

        BuildRule rule = new BuildRule();

        String dir = new File(_schema).getParent();
        
        // add sources
        rule.addSource(_schema);
        
        // add targets
        StringTokenizer targets = new StringTokenizer(_targets, ", ");
        while (targets.hasMoreTokens()) {
            String name = targets.nextToken().trim();
            String target;
            String targetDir;
            if (_destDir != null) {
                targetDir = _destDir + File.separator + 
                    _targetPackage.replace('.', File.separatorChar);
            } else if (_targetPackage != null) {
                targetDir = _targetPackage.replace('.', File.separatorChar);
            } else {
                targetDir = dir;
            }
            target = targetDir + File.separator + name;

            rule.addTarget(target + SOURCE_EXT);
            rule.addTarget(target + DESCRIPTOR_EXT);
        }

        // add options
        rule.addOption(PACKAGE, _targetPackage);
        rule.addOption(TYPES, _types);
        rule.addOption(DEST_DIR, _destDir);
        
        return rule;
    }

    /**
     * Returns the command line arguments to be passed to Castor
     * to build the targets.
     */
    public String getArgs() {
	StringBuffer args = new StringBuffer();
        args.append("-i ");
        args.append(_schema);

        if (_targetPackage != null) {
            args.append(" -package ");
            args.append(_targetPackage);
        }

        if (_destDir != null) {
            args.append(" -dest ");
            args.append(_destDir);
        }

        if (_types != null) {
            args.append(" -types ");
            args.append(_types);
        }

        args.append(" -f "); // turns off prompting if the targets exist

	return args.toString();
    }

} // End CastorBuildRule
