/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2000 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 */
 
package org.exolab.tools.ant;


import java.io.File;
import java.util.StringTokenizer;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;


/**
 * Task to clean targets of sources.
 * This task accepts the following arguments:
 * <ul>
 * <li>sources
 * <li>optDir
 * </ul>
 * Of these arguments, only <b>sources</b> is required.
 * <p>
 * When this task executes, it cleans all targets associated with the paths
 * listed in <b>sources</b>. <br>
 * To do this, each path <i>X</i> must have an associated <i>.opt</i>
 * file which contains the paths of targets generated from <i>X</i>.<br>
 * The <i>.opt</i> file is read from <i>&lt;optDir&gt;/X.opt</i> if 
 * <b>optDir</b> is set, or <i>X.opt</i> if unset.<br>
 * The <i>.opt</i> file must be a serialized instance of
 * <b>org.exolab.tools.ant.BuildRule</b>
 * <p>
 *
 * @version     $Revision: 1.1.1.1 $ $Date: 2002/01/22 00:21:37 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         org.exolab.tools.ant.BuildRule
 */
public class CleanTargets extends Task
{
    /**
     * A comma or space separated list of sources to clean the targets of
     */
    private String _sources = null;

    /**
     * The directory where the .opt files are located
     */
    private String _optDir = null;


    /**
     * Set the paths of the source files whose targets need to be cleaned
     *
     * @param sources a comma or space separated list of sources
     */
    public void setSources(String sources) {
        _sources = sources;
    }

    /**
     * Set the directory where the .opt files are located
     *
     * @param dir the directory where the .opt files are located
     */
    public void setOptDir(String dir) {
        _optDir = dir;
    }

    /**
     * Execute this task
     *
     * @throws BuildException if 'sources' hasn't been set
     */
    public void execute() 
	throws BuildException {

        if (_sources == null) {
            throw new BuildException("'sources' not set");
        }
        StringTokenizer paths = new StringTokenizer(_sources, ", ");
        while (paths.hasMoreTokens()) {
            String path = paths.nextToken().trim();
            clean(path);
        }
    }

    /**
     * Clean all derived files associated with source
     *
     * @param source the source path
     */
    private void clean(String source)
        throws BuildException
    {
        File opt = null;
        if (_optDir != null) {
            opt = new File(BuildRule.path(_optDir, source));
        } else {
            File src = new File(source);
            opt = new File(BuildRule.path(src.getParent(), src.getName()));
        }

        BuildRule rule = BuildRule.read(opt.getPath());
        if (rule != null) {
            project.log("Deleting targets of: " + source);
            BuildRuleHelper.clean(rule, project);
        } else {
            project.log("Build rule does not exist: " + opt.getPath(),
                        project.MSG_VERBOSE);
        }

        if (opt.exists()) {
            project.log("Deleting: " + opt.getPath(), project.MSG_VERBOSE);
            opt.delete();
        }
    }

} // End CleanTargets
